
/* 
 * SQULView.m -- Implementation file for the SQULView class 
 *
 * You may freely copy, distribute, and reuse the code in this example.
 * NeXT disclaims any warranty of any kind, expressed or implied, as to 
 * its fitness for any particular use, nor even that it has a use.
 *
 */

#import "SQULView.h"
#import "drawingFuncs.h"
#import "nxlcau2s.h"
#import <math.h> 
#import <stdlib.h>
#import <string.h>

#define MB 12				/* maximum binomial coefficient */
#define NG 80				/* number of Golay Classes	*/

extern char ascrule[KK][KK][KK][KK][KK][KK][KK][KK];
extern char auxrule[KK][KK][KK][KK][KK][KK][KK][KK];
extern char golRule[];			/* evolutionary rule */
extern char ulrule[];			/* evolutionary rule */
extern char ttrule[];			/* evolutionary rule */

extern char    arr1[], arr2[];
       int             brr2[AL];
extern double  bp[];			/* Bernstein coefficients    */
extern int     tus[TS];			/* totalistic use stats    */
extern int     lf[];			/* number of live cells	     */
       int     llf[KK];			/* save number of live cells */

extern NXColor colst[KK];    		/* Array for state colors    */

int    checkULBoard=NO;
int    drawULBack=YES;
int    drawULBnstein=NO;
int    drawULMosaic=NO;
int    drawULFreqs=NO;
int    drawULPlane=NO;
int    drawULRturn=NO;
int    drawSalami=NO;
int    drawULSmall=NO;
int    drawULTrajec=NO;
int    slowULFreqs=NO;
int    smallULSmall=NO;
int    smallULMosaic=NO;

int    lastULPlane=0;			/* 0 = none, 1 = evol, 2 = mosaic */
int    lastULTraj=0;			/* 0 = none, 1 = traj, 2 = return */
int    moULTiles=3;
int    pQULDots=NO;
int    trajULLen=10;
int    repULPlEv;
int    rturnULNum=1;

int    ULgtb[NG] = {0,1,2,2,3,4};

extern int    drawview;
extern int    co, dn;

void    argfram(int n, int l);
void    argdiag(int n, int l);
void    ulonegen(char *aa, int *bb, int l, int m, int n); 
void    asctobin();
void	asctoqul();
void	asczero();
void    bargraf(int m, int n);
void    ulbdot(int i, int j, int l);
void    ulbline(double x, double y, int l);
int     istot();
int     ix2(int i, int j);
int     ix3(int i, int j, int k);
int     nearest(double x, double y);
void    onegen(int n);
void    ranblin(double p1, double p2);
void    renewarry();
void    savearry();
void    screen(char *aa, int l, int m, int n);
void    statecount(char *aa, int l, int m, int n);
void    suggULrule(int i);
void    videodot(int i, int j, int l);
void    wqusscrrul(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK]);
void    wrru(char *c, char ru[KK][KK][KK][KK][KK][KK][KK][KK]);
void    zerline();

@implementation SQULView
 
/* -------------------------------- S Q U L -------------------------------- */

/* put up the principal window for the SQULView. Since squl.nib is a 	  */
/* secondary .nib file, it is not loaded until there is a demand, which   */
/* would usually arise from the "CHIL" option in the main menu.	   	  */

- loadSqul:sender {
  if (squlWindow==nil) {
    if (![NXApp loadNibSection: "squl.nib" owner: self withNames: NO]) 
	{return nil;}
    }
  dn=3;
  [SqulWindow makeKeyAndOrderFront: sender];
  [squlWindow orderFront: nil];
  [self copyRule: sender];
  if (istot()) [self copyTRule: sender];
return self;
}

void drawULsquare() {int i, j;
  for (i=0; i<128; i++) for (j=0; j<128; j++) videodot(j,i,arr1[i*128+j]+1);
}

void drawULcube() {int i, j, k;
  for (i=0; i<32; i++) for (j=0; j<32; j++) for (k=0; k<16; k++) 
    videodot(j+32*(k/4),i+32*(k%4),arr1[ix3(i,j,k)]+1);
}

void manyULsquare(i,j) int i, j; {int k, l;
  for (k=0; k<128; k++) for (l=0; l<128; l++)  
    videodot(k+128*i,l+128*j,arr1[ix2(k,l)]+1);
}

void drawULgrid() {int i;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<5; i++) {
  PSmoveto((double)(128.0*i),0.0); PSlineto((double)(128.0*i),512.0);}
  for (i=0; i<5; i++) {
  PSmoveto(0.0,(double)(128.0*i)); PSlineto(512.0,(double)(128.0*i));}
  PSstroke();
}

- printSQUL:sender {
   if (keyWindow==SqulWindow) 
	{[SqulWindow smartPrintPSCode: self];}
   else if (keyWindow==planeWindow) {
    switch (lastULPlane) {
      case 0: break;
      case 1: drawULPlane=YES;   break;
      case 2: drawULMosaic=YES;  break;
      case 3: drawULSmall=YES;   break;
      case 4: smallULMosaic=YES; break;
      default: break;}
    [planeWindow smartPrintPSCode: self];
    switch (lastULPlane) {
      case 0: break;
      case 1: drawULPlane=NO;   break;
      case 2: drawULMosaic=NO;  break;
      case 3: drawULSmall=NO;   break;
      case 4: smallULMosaic=NO; break;
      default: break;}
    }
return self;
}

- windowDidBecomeKey:sender {
	keyWindow=sender;
	[menuCopy setTarget: self];
	[menuCopy setAction: @selector(copySQUL:)];
	[menuCopy setEnabled: YES];
	[menuPrint setTarget: self];
	[menuPrint setAction: @selector(printSQUL:)];
	[menuPrint setEnabled: YES];
return self;
}

- windowDidResignKey:sender {
	[menuCopy setEnabled: NO];
	[menuPrint setEnabled: NO];
return self;
}

- copySqulView:sender {
NXRect zbounds;
id     pBoard;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [SqulView getBounds: &zbounds];
  [SqulView writePSCodeInside: &zbounds to: pBoard]; 
return self;
}

- copyWhoosit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawview=13;
  drawULBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [trajView getBounds: &zbounds];
  [trajView writePSCodeInside: &zbounds to: pBoard]; 
  drawULBack=YES;
  *flag = NO;
return self;
}

- copyWhatsit: (int *) flag {
NXRect zbounds;
id     pBoard;
  *flag = YES;
  drawULBack=NO;
  pBoard = [Pasteboard new];
  [pBoard declareTypes: &NXPostScriptPboardType num: 1 owner: self];
  [planeView getBounds: &zbounds];
  [planeView writePSCodeInside: &zbounds to: pBoard]; 
  drawULBack=YES;
  *flag = NO;
return self;
}

- copySQUL:sender {
   savearry();
   if (keyWindow==SqulWindow) {[self copySqulView: self];}
   else if (keyWindow==trajWindow) switch (lastULTraj) {
          case 0: break;
          case 1: [self copyWhoosit: &drawULTrajec]; break;
          case 2: [self copyWhoosit: &drawULRturn];  break;
          case 3: [self copyWhoosit: &drawULFreqs];  break;
          case 4: [self copyWhoosit: &slowULFreqs];  break;
          default: break;}
   else if (keyWindow==planeWindow) switch (lastULPlane) {
        case 0: break;
        case 1: [self copyWhatsit: &drawULPlane];   break;
        case 2: [self copyWhatsit: &drawULMosaic];  break;
        case 3: drawview=7;  [self copyWhatsit: &drawULSmall];   break;
        case 4: drawview=11; [self copyWhatsit: &smallULMosaic]; break;
        default: break;}
   renewarry();
return self;
}

- drawPlEvol:sender {int i;
  repULPlEv=[[sender selectedCell] tag];
  drawview=7;
  drawULPlane=YES;
  asctobin();
  if (repULPlEv==0) [self display]; else for (i=0; i<repULPlEv; i++) {
     [genEv setIntValue: i];
     onegen(AL);
     [self display];
     }
  [genEv setIntValue: repULPlEv];
  drawULPlane=NO;
  lastULPlane=1;
return self;
}

- printPlEvol:sender {
    repULPlEv=0;
    drawview=7;
    drawULPlane=YES;
    [keyWindow smartPrintPSCode: self];
    drawULPlane=NO;
return self;
}

- drawMosaic:sender {
    drawview=11;
    drawULMosaic=YES;
    [self display];
    drawULMosaic=NO;
    lastULPlane=2;
return self;
}

- printMosaic:sender {
    drawview=11;
    drawULMosaic=YES;
    [keyWindow smartPrintPSCode: self];
    drawULMosaic=NO;
return self;
}

- showProbability:sender {int i; double pp, p[KK];
  for (i=0; i<KK; i++) p[i]=[[sender cellAt: i: 0] doubleValue];
  for (pp=0, i=0; i<KK; i++) pp+=p[i];
  for (i=0; i<KK; i++) {
    [[pSliders cellAt: i: 0] setDoubleValue: p[i]/pp];
    [[pWindows cellAt: i: 0] setDoubleValue: p[i]/pp];
    }
return self;
}

- randomLine:sender {double p[KK]; int i;
  for (i=0; i<KK; i++) p[i]=[[pSliders cellAt:i:0] doubleValue];
  ranblin(p[0],p[1]);
return self;
}

- sCellLine:sender {
  smallULSmall=YES;
  [planeView display];
  smallULSmall=NO;
return self;
}

- uniformLine:sender {ranblin(0.5,0.5); return self;}

- checkerBoard:sender {
  checkULBoard=YES;
  [planeView display];
  checkULBoard=NO;
return self;
}

- uprule:sender {suggULrule(0); [self anyrule: self]; return self;}
- lerule:sender {suggULrule(1); [self anyrule: self]; return self;}
- dorule:sender {suggULrule(3); [self anyrule: self]; return self;}
- rirule:sender {suggULrule(2); [self anyrule: self]; return self;}
- idrule:sender {suggULrule(4); [self anyrule: self]; return self;}
- mjrule:sender {suggULrule(5); [self anyrule: self]; return self;}

- anyrule:sender {
   [self copyRule: self];
   [rulePanel setAllRules: self];
return self;}

void suggULrule(i) int i; {int j, i0, i1, i2, i3, i4, i5, i6, i7;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++) {
    switch (i) {
      case 0: j=i2; break;
      case 1: j=i3; break;
      case 2: j=i1; break;
      case 3: j=i0; break;
      case 4: j=i4; break;
      case 5: j=0; if (i0+i1+i2+i3+i4+i5+i6+i7>4) j=1; break;
      default: j=0; break;}
    ascrule[i0][i1][i2][i3][i4][i5][i6][i7] = '0'+j;
    }
}

/* --------------------------- E V O L U T I O N --------------------------- */

- drawSmEvol:sender {int k;
    drawview=7;
    drawULSmall=YES;
    asctoqul();
    if ([reStore state]) savearry();
    sscanf([[sender selectedCell] title],"%d",&repULPlEv);
    [self display];
    if (repULPlEv>0) for (k=0; k<repULPlEv; k++) [self display];
    if ([reStore state]) renewarry();
    drawULSmall=NO;
    lastULPlane=3;
return self;
}

- printSmEvol:sender {
    drawview=7;
    drawULSmall=YES;
    repULPlEv=0;
    [keyWindow smartPrintPSCode: self];
    drawULSmall=NO;
return self;
}

- drawSmMosaic:sender {
    moULTiles=[moTies selectedRow]+2;
    drawview=11;
    smallULMosaic=YES;
    asctoqul();
    savearry();
    [self display];
    renewarry();
    smallULMosaic=NO;
    lastULPlane=4;
return self;
}

- printSmMosaic:sender {
    drawview=11;
    smallULMosaic=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    smallULMosaic=NO;
return self;
}

- drawTraj:sender {
    sscanf([[sender selectedCell] title],"%d",&trajULLen);
    pQULDots=[probDots state];
    drawview=13;
    drawULTrajec=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULTrajec=NO;
    lastULTraj=1;
return self;
}

- printTraj:sender {
    drawview=13;
    drawULTrajec=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULTrajec=NO;
return self;
}

- drawRtrn:sender {
    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    drawULRturn=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULRturn=NO;
    lastULTraj=2;
return self;
}

- printRtrn:sender {
    drawview=13;
    drawULRturn=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULRturn=NO;
return self;
}

- drawFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    drawULFreqs=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    drawULFreqs=NO;
    lastULTraj=3;
return self;
}

- printFreq:sender {
    drawview=13;
    drawULFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    drawULFreqs=NO;
return self;
}

- slowFreq:sender {
//    sscanf([[sender selectedCell] title],"%d",&rturnULNum);
    drawview=13;
    slowULFreqs=YES;
    asctoqul();
    savearry();
    [trajView display];
    renewarry();
    slowULFreqs=NO;
    lastULTraj=4;
return self;
}

- printSlowFreq:sender {
    drawview=13;
    slowULFreqs=YES;
    savearry();
    [keyWindow smartPrintPSCode: self];
    renewarry();
    slowULFreqs=NO;
return self;
}

/* draw rectangle (i,j) in a mosaic of (u,v) */
/* whose individual rectangles measure (m,n) */
void manyULrect(i,j,l,m,n,q) int i, j, l, m, n, q; {int ii, jj;;
  asctoqul();
  for (ii=0; ii<l; ii++) for (jj=0; jj<m; jj++) 
	videodot(i*l+ii,j*m+jj,arr1[ii+l*(jj+m*(q%n))]);
  ulonegen(arr1,brr2,l,m,n);
}

/* m = high/moULTiles, n = wide/moULTiles */
void smallULgrid(m,n) int m, n; {int i; double em, en, mo;
  em=(double)m;
  en=(double)n;
  mo=(double)moULTiles;
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
  for (i=0; i<moULTiles+1; i++) {
  PSmoveto((double)(en*i),0.0); PSlineto((double)(en*i),mo*em);}
  for (i=0; i<moULTiles+1; i++) {
  PSmoveto(0.0,(double)(em*i)); PSlineto(mo*en,(double)(em*i));}
  PSstroke();
}

/* -----------------------  D E F I N E   R U L E S  ----------------------- */

/* - - - - - - - -   B E R N S T E I N   H I S T O G R A M S   - - - - - - */

- drawBern:sender {
    drawview=12;
    drawULBnstein=YES;
    [self display];
    drawULBnstein=NO;
    [window setDocEdited: YES];
return self;
}

- selfBern:sender {
    drawview=12;
    drawULBnstein=YES;
    [self display];
    drawULBnstein=NO;
    [window setDocEdited: YES];
return self;
}

- printRule:sender {
   drawview=12;
   drawULBnstein=YES;
   [keyWindow printPSCode:self];
   drawULBnstein=NO;
return self;
}

- readRule:sender 
    {[rulePanel readWRule: sender]; [self copyRule: self]; return self;}

- enterRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

      asczero(ascrule);
///***///
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqulWindow setDocEdited: NO];
     if (istot()) [ruleType setStringValue:"totalistic"];
     else [ruleType setStringValue:""];
return self;
}

- resetRule:sender {int i, j; 
    for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
	[[theOMatrix cellAt: j: i] setIntValue: NO];
	[[theZMatrix cellAt: j: i] setIntValue: NO];
	}
return self;
}

- copyRule:sender {char ch[WNML]; int i;
///***///
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     wqusscrrul(ch,ascrule);
     [xsRuleNumber setStringValue: ch];
     if (istot()) {
	[tRuleNumber setIntValue: totonu(0)];
	for (i=0; i<TS; i++) 
	  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
	}
     else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqulWindow setDocEdited: NO];
     [bernView drawBern: nil];
return self;
}

- readTRule:sender 
    {[rulePanel readTRule: sender]; [self copyRule: self]; return self;}

- enterTRule:sender {char ch[WNML];
char    tiempo[30], fecha[30];
struct  tm  *timeptr;
time_t  secsnow;

   time(&secsnow);
   timeptr = localtime(&secsnow);
   sprintf(tiempo,"%02d:%02d:%02d\n",
	timeptr->tm_hour, 
	timeptr->tm_min,
        timeptr->tm_sec);
   [tiTime setStringValue: tiempo];
   sprintf(fecha,"%d-%d-19%02d\n",
	((timeptr->tm_mon) + 1), 
	timeptr->tm_mday,
        timeptr->tm_year);
   [tiDat setStringValue: fecha];

     ttrule[[sender selectedCol]] = [[sender selectedCell] state]+'0';
     totoasc();
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     [rulePanel setAllRules: self];
     [SqulWindow setDocEdited: NO];
     if (istot()) [tRuleNumber setIntValue: totonu(0)];
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [bernView drawBern: self];
return self;
}

- copyTRule:sender {char ch[WNML]; int i;
     wrru(ch,ascrule);
     [ruleNumber setStringValue: ch];
     if (istot()) {
		[tRuleNumber setIntValue: totonu(0)];
		for (i=0; i<TS; i++) 
		  [[theToMatrix cellAt: 0: i] setIntValue: ttrule[i]-'0'];
		}
       else [tRuleNumber setStringValue: "enter totalistic #"];
     [SqulWindow setDocEdited: NO];
     [squlView selfBern: nil];
return self;
}

void berncoeff() {int i, i0, i1, i2, i3, i4, i5, i6, i7;
  for (i=0; i<BD; i++) bp[i]=0.0;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
  for (i7=0; i7<KK; i7++)
    if (ascrule[i0][i1][i2][i3][i4][i5][i6][i7]=='1')
      bp[i0+i1+i2+i3+i4+i5+i6+i7]+=1.0;
}

/* -------------------------------- * * * * -------------------------------- */

- initFrame:(const NXRect *)frameRect
/*
 * Initializes the new SQULView object.  First, an initFrame: message is sent 
 * to super to initialize SQULView as a View.  Next, the SQULView sets its own 
 * state -- that it is opaque and that the origin of its coordinate system lies 
 * in the center of its area.  It then creates and initializes its associated 
 * objects, a Storage object, an NXCursor, and a Cell.  Finally, it loads into 
 * the Window Server some PostScript procedures that it will use in drawing 
 * itself. 
 */
{
    NXPoint	spot;
	
    [super initFrame:frameRect];
/*  [self translate:floor(frame.size.width/2) :floor(frame.size.height/2)]; */
    [self setOpaque:YES];
    crossCursor=[NXCursor newFromImage:[NXImage newFromSection:"cross.tiff"]];
    spot.x = spot.y = 7.0;
    [crossCursor setHotSpot:&spot];

  PSsetgray(NX_BLACK);
  PSselectfont("Times-Roman",16.0);

  [self setFlipped: YES];
  drawview=7;			      /* prefer origin in upper left corner */
	return self;
}

- setDelegate:anObject
/*
 * Sets the SQULView's delegate instance variable to the supplied object.
 */
{
	delegate = anObject;
	return self;
}

- drawSelf:(const NXRect *)rects :(int)rectCount
/*
 * Draws the SQULView's background and axes.  If there are any points,
 * these are drawn too.
 *
 * (Note:  For simplicity, although SQULView only repaints the background 
 * of the update region, it redraws the entire axes.  For better performance, 
 * only those parts of the axes that fall within the update region should 
 * be redrawn.)   
 */
{
	int		 i, j, k, l, n, ii, jj;
        double           p, q, r, s, t, mo;
	NXRect		 plotRect;

	if (rects == NULL) return self;

	/* paint visible area white then draw axes */
	PSsetgray(NX_WHITE);
	if (drawULBack) NXRectFill(&rects[0]);
	PSsetgray(NX_DKGRAY);

	if (drawULBnstein==YES) {
//	  for (i=0; i<TS; i++) bp[i]=0.0;
//	  for (i=0; i<NG; i++) for (j=0; j<MB; j++) {
//		if ([[theZMatrix cellAt: j: i] state]) bp[ULgtb[i]]+=1.0;
//		if ([[theOMatrix cellAt: j: i] state]) bp[ULgtb[i]+1]+=1.0;
//		}
	berncoeff();
	  ulgfram(50,3);
	  ulgdiag(50,3);
	  bulgraf(TS-1,50);
	  }

	if (drawULTrajec==YES) {
	  ulgfram(75,3);
	  ulgdiag(75,3);
	  bulgraf(TS-1,75);
	  statecount(arr1,16,16,16);
	  for (j=0; j<KK; j++) llf[j]=lf[j];
	  ulonegen(arr1,brr2,16,16,16);
	  if (pQULDots) ulbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	    else ulbline(0.034*((double)lf[1]),0.034*((double)llf[1]),0);
	  for (i=0; i<trajULLen; i++) {
	    for (j=0; j<KK; j++) llf[j]=lf[j];
	    ulonegen(arr1,brr2,16,16,16);
	    if (pQULDots) ulbdot((75*llf[1])/4096,(75*lf[1])/4096,3);
	  else ulbline(0.034*((double)llf[1]),0.034*((double)lf[1]),1);
// printf("(%6.3f, %6.3f)\n",0.034*((double)lf[1]),0.034*((double)llf[1]));
	    }
	  PSstroke();
	  }

	if (drawULRturn==YES) {
	  ulgfram(75,3);
	  ulgdiag(75,3);
	  bulgraf(TS-1,75);
	  for (i=0; i<75; i++) {
	    p=((double)i)/75.0; q=1.0-p;
	    ranblin(q,p);
	    for(j=0; j<rturnULNum; j++) ulonegen(arr1,brr2,16,16,16);
	    ulbdot(i,(75*lf[1])/4096,3);
	    }
	  }

	if (drawULFreqs==YES) {
	  ulgfram(75,3);
	  for (i=0; i<75; i++) {
	    ulonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) ulbdot(i,k++,j%3);
	    ulbdot(i,(75*lf[1])/4096,3);
	    }
	  }

	if (slowULFreqs==YES) {
	  ulgfram(75,3);
	  for (i=0; i<75; i+=3) {
	    ulonegen(arr1,brr2,16,16,16);
	    for (j=0, k=0; j<TS; j++)
		if (tus[j]>57) for (l=0; l<tus[j]; l+=58) {
		  ulbdot(i,k,j%3);
		  ulbdot(i+1,k,j%3);
		  ulbdot(i+2,k,j%3);
		  k++;
		  }
	    ulbdot(i,(75*lf[1])/4096,3);
	    ulbdot(i+1,(75*lf[1])/4096,3);
	    ulbdot(i+2,(75*lf[1])/4096,3);
	    }
	  }

        if (checkULBoard==YES) {
   	  [self getBounds: &plotRect];
	  mo=(double)moULTiles;
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=0; k<i/2; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=0;
            for (l=j/2; l<j; l++) arr1[k*j+l]=1;
            }
          for (k=i/2; k<i; k++) {
	    for (l=0; l<j/2; l++) arr1[k*j+l]=1;
            for (l=j/2; l<j; l++) arr1[k*j+l]=0;
            }
	  }

        if (smallULSmall==YES) {
	  mo=(double)moULTiles;
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  zerline();
          for (k=i/3; k<2*(i/3); k++) for (l=j/3; l<2*(j/3); l++) 
		arr1[k*j+l]=random()%KK;
	  }

        if (drawULSmall==YES) {
	  [window setTitle: "evolution"];
   	  [self getBounds: &plotRect];
	  j=(int)(plotRect.size.width/4.0);           /* 4.0 for drawview=7 */
	  i=(int)(plotRect.size.height/4.0);
	  if (j*i*i < AL) {
	    ulscreen(arr1,j,i,i,2);
	    if(repULPlEv>0) ulonegen(arr1,brr2,j,i,i);
	    } else printf("volume = %6d\n",j*i*i);
	  }

        if (smallULMosaic==YES) {
	  mo=(double)moULTiles;
	  [window setTitle: "mosaic"];
   	  [self getBounds:&plotRect];
	  j=(int)(plotRect.size.width/mo);
	  i=(int)(plotRect.size.height/mo);
	  if (i*j*j < AL) {
	    for (k=0; k<moULTiles; k++) for (l=0; l<moULTiles; l++) {
	      manyULrect(k,l,i,j,j,3);
	      [window flushWindow];
	      }
	  smallULgrid(i,j);
	    } else printf("volume = %6d\n",i*j*j);
	  }

        if (drawSalami==YES) {
	  mo=(double)moULTiles;
	  [window setTitle: "slices of a 5x5x5 cube"];
	  for (i=0; i<5; i++) for (j=0; j<5; j++) 
	    for (ii=0; ii<25; ii++) for (jj=0; jj<25; jj++) 
	      videodot(i*25+ii,j*25+jj,arr1[ii+25*(jj+25*(i+5*j))]);
	  PSsetgray(NX_BLACK); 
	  PSsetlinewidth(0.1);
	  for (i=0; i<6; i++) {
	  PSmoveto((double)(50*i),0.0); PSlineto((double)(50*i),250.0);}
	  for (i=0; i<6; i++) {
	  PSmoveto(0.0,(double)(50*i)); PSlineto(250.0,(double)(50*i));}
	  PSstroke();
	  }

        if (drawULPlane==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) Plane Evolution"];
	drawULsquare();
	break;
     case 3: 
	[window setTitle: "3-D Solid Tetrahedron Evolution"];
	drawULcube();
	drawULgrid();
	break;
     default: [window setTitle: "SQUL2S Plane Evolution"];break;}
	}

        if (drawULMosaic==YES) {
   switch (dn) {
     case 2: 
	[window setTitle: "von Neumann (2,1) manysquare Plane Evolution"];
	for (i=0; i<4; i++) for (j=0; j<4; j++) {
	  manyULsquare(i,j);
	  onegen(AL);
	  [window flushWindow];
	  }
	drawULgrid();
	break;
     default: [window setTitle: "SQUL2S Plane Evolution"];break;}
	}

	return self;
}

- sizeTo:(NXCoord)width :(NXCoord)height
/*
 * Ensures that whenever the SQULView is resized, the origin of its 
 * coordinate system is repositioned to the center of its area.
 */
{
//	  [self setFlipped: YES];
	[super sizeTo: width: height];
	if (drawview==4||drawview==6||drawview==7||drawview>=10) 
		{[self setDrawOrigin:0 :0]; return self;}
	else
		[self setDrawOrigin: -floor(width/2) : -floor(height/2)]; 
	return self;
}

- mouseDown:(NXEvent *) theEvent
/*
 * Responds to a message the system sends whenever the user presses the mouse
 * button when the cursor is over the SQULView.  The SQULView changes the 
 * cursor to a cross-hairs image and then starts asking for mouse-dragged or 
 * mouse-up events.  As it receives mouse-dragged events, the SQULView updates 
 * the readOut text Cell with the cursor's coordinates.  If the user releases 
 * the mouse button while the cursor is over the SQULView, the SQULView 
 * registers the point and then sends a message to its delegate notifying it 
 * of the new point.
 */
{
	int	looping = YES, oldMask;
	NXPoint	aPoint;
	NXRect	plotRect;

      /*   double  x, y,  eks, wye; */
	 double ex, wy;
	 int k;

	[crossCursor set];
	oldMask = [window addToEventMask:NX_MOUSEDRAGGEDMASK];
 				[window setDocEdited:YES];
		                aPoint = theEvent->location;
				aPoint.y = aPoint.y;
		                [self convertPoint:&aPoint fromView:nil];
                                ex=aPoint.x;
                                wy=aPoint.y;
				k=nearest(ex,wy);

//       if (theEvent->data.mouse.click==3) {} else 
//       if (theEvent->data.mouse.click==2) {} else 
	{
	[self lockFocus];
//         PSsetinstance(YES);
	do {
// 		PSnewinstance();
		aPoint = theEvent->location;
		[self convertPoint:&aPoint fromView:nil];
		aPoint.y= (plotRect.size.height-1)-aPoint.y;
                ex=aPoint.x;
                wy=aPoint.y;
				PSsetgray(NX_BLACK);
		[window flushWindow];
		if (theEvent->type == NX_MOUSEUP) {
	/* on mouse-up, register point, inform delegate, and clean up state */
			[window flushWindow];
			looping = NO;
		}	
   	} while (looping && (theEvent = 
		[NXApp getNextEvent:NX_MOUSEUPMASK|NX_MOUSEDRAGGEDMASK]));
	[self unlockFocus];

	}

	[window setEventMask:oldMask];
	[NXArrow set];
	return self;
}

@end

/* end SQULView.m */
