
/* HEXL21.C					*/
/* Two-Dimensional Hexagonal Life		*/
/* two-state cells with six neighbors		*/
/*						*/
/* Reference:					*/
/*						*/
/* Harold V. McIntosh, 16 May 1987		*/
/* 7 July 1987 - separate rules for each state	*/
/* 8 July 1988 - binary, 3 neighborhood classes	*/
/* 9 July 1988 - hexagonal lattice 		*/

/* parameters related to arrays and automaton characteristics	*/

# define QL	    80  /* edge of square of evolution		*/
# define KK	     2  /* number of states per cell		*/
# define BPD	     7	/* degree of Bernstein polynomial	*/
# define GK        128	/* KK^BPD= total number neighborhoods	*/
# define GH      GK/KK	/* GK/KK= 1/KKth no. of neighborhoods	*/
# define NWOC	    13  /* number of symmetry classes w/o ctr	*/
# define TWIC	 NWOC*KK  /* number of symmetry classes w/ cntr	*/

/* coordinates of significant objects on the display screen 	*/

# define NRR	     3  /* row coordinate for neighborhood disp	*/
# define NRC	    25	/* column coordinate for neighborhood d	*/		
# define TRR	     2  /* row coordinate for totalistic rule	*/
# define TRC	     0	/* column coordinate for t'listic  rule	*/		
# define WRR	     0  /* row coordinate for Wolfram number	*/
# define WRC	     8	/* column coordinate Wolfram number	*/		
# define AUR	    64  /* row (from bottom) for planar array	*/
# define AUC	    60  /* column for planar array		*/
# define BER	     0  /* row (from bottom) for Bernstein p.	*/
# define BEC	   219  /* column for Bernstein Polynomial	*/
# define PLR	    60  /* row for planar array			*/
# define PLC	     0  /* column for planar array		*/

# define lcell(i,j) arrr1[li+i][lj-(j)]=1

extern char   ascrule[KK][KK][KK][KK][KK][KK][KK];
extern char   auxrule[KK][KK][KK][KK][KK][KK][KK];

char    trule[TWIC];	/* evolutionary rule */
char    arule[TWIC];	/* auxiliary rule flags */
int     li, lj;		/* cursor in cell array */
int     nr[NWOC];	/* nbrhd representative of symm class */
int     rs[TWIC];	/* rule use statistics */
int     gc[GK];		/* Golay code key */
int     wc[BPD+1];	/* Bernstein coefficients */
int     ix0, iy0;	/* coordinates for pen in de B. */
double  bp[BPD+1];	/* Bernstein coefficients */

mainhexl(c) char c; {
int  i, j, ln;
int  arrr1[QL][QL];

    li=QL/2; lj=QL/2;

    for (i=0; i<TWIC; i++) rs[i]=0;
    for (i=0; i<TWIC; i++) arule[i]=' ';

  golta();
  for (j=1; j<KK; j++) for (i=0; i<GH; i++) gc[i+j*GH]=gc[i]+j*NWOC;
  for (i=0; i<NWOC; i++) bp[i]=((double)wc[i]);

    conlif();

    hscrrul();
    wscrrul();
    hrmenu();
    phrame(3);
    rancen(arrr1);
    screen(arrr1);
    phrame(3);

    while (0<1) {			/* set up one run */
    hrmenu();
    hscrrul();
    wscrrul();
    if (c=='q') break;
    switch (c) {
	case 'g':			/* one cycle of evolution */
	    phrame (1);
	    onehgen(arrr1);
	    screen(arrr1);
	    break;
        case ':':			/* multiple cycles of evolution */
	    i=0;
	    rstat();
	    phrame (1);
            do {
	      i++;
	      videocursor(0,8,0);
	      printf("%5d",i);
	      onehgen(arrr1);
	      screen(arrr1);
	      rstat();
	      } while (!kbdst());
            break;
    case 'M':				/* record rule usage */
      phrame (1);
      videoscroll(18,0,24,25,0,0);
      for (ln=0; ln<40; ln++) {
        for (i=0; i<TWIC; i++) rs[i]=0;
        videocursor(0,8,0);
        printf("%5d",ln+1);
        onehgen(arrr1);
        screen(arrr1);
        rinci(40-ln);
        if (kbdst()) {kbdin(); break;};
	};
      break;
	case 's':
	    rstat();
	    break;
	case 't':			/* probability plot */
	    for (i=0; i<BPD+1; i++) bp[i]=0.0;
	    if (trule[0]=='1') bp[0]+=1.0;
	    if (trule[1]=='1') bp[1]+=6.0;
	    if (trule[2]=='1') bp[2]+=6.0;
	    if (trule[3]=='1') bp[2]+=6.0;
	    if (trule[4]=='1') bp[2]+=3.0;
	    if (trule[5]=='1') bp[3]+=6.0;
	    if (trule[6]=='1') bp[3]+=12.0;
	    if (trule[7]=='1') bp[3]+=2.0;
	    if (trule[8]=='1') bp[4]+=6.0;
	    if (trule[9]=='1') bp[4]+=6.0;
	    if (trule[10]=='1') bp[4]+=3.0;
	    if (trule[11]=='1') bp[5]+=6.0;
	    if (trule[12]=='1') bp[6]+=1.0;
	    if (trule[13]=='1') bp[1]+=1.0;
	    if (trule[14]=='1') bp[2]+=6.0;
	    if (trule[15]=='1') bp[3]+=6.0;
	    if (trule[16]=='1') bp[3]+=6.0;
	    if (trule[17]=='1') bp[3]+=3.0;
	    if (trule[18]=='1') bp[4]+=6.0;
	    if (trule[19]=='1') bp[4]+=12.0;
	    if (trule[20]=='1') bp[4]+=2.0;
	    if (trule[21]=='1') bp[5]+=6.0;
	    if (trule[22]=='1') bp[5]+=6.0;
	    if (trule[23]=='1') bp[5]+=3.0;
	    if (trule[24]=='1') bp[6]+=6.0;
	    if (trule[25]=='1') bp[7]+=1.0;
	    gphram(219,0,100);
	    ghdiag(219,0,100);
	    bhgraf(219,0,BPD,100);
	    break;
	case 'l':				/* edit cell array */
	    edhline(arrr1);
	    break;
        case 'r':				/* edit rule */	
	    edhrule();
	    videocursor(0,4,0);
	    break;
	case 'd':				/* de Bruijn diagram */
//	    edrui();
	    break;
       case 'u':				/* sparse init arry */
	    zarry(arrr1);
	    arrr1[QL/4][QL/4]=1;
	    arrr1[QL/2+1][QL/2]=1;
	    arrr1[QL/2-1][QL/2]=1;
	    arrr1[QL/2][QL/2+1]=1;
	    arrr1[QL/2][QL/2-1]=1;
	    screen(arrr1);
	    break;
	case 'v':					/* Golay's Life */
	    conlif();
	    break;
	case 'X':					/* random rule */
	    ranhrul();
	    break;
	case 'x':					/* random rule */
	    for (i=0, j=0; j<TWIC; j++) {
	      if (i==0) i=rand();
	      if (arule[j]==' ') {trule[j]='0'+i%KK; i/=KK;};
	      };
	    break;
	case 'y':					/* random line */
	    rancen(arrr1);
	    screen(arrr1);
	    break;
	case 'Y':					/* random line */
	    ranarr(arrr1);
	    screen(arrr1);
	    break;
/*	case '=':
/*	    for (i=1; i<8;  i++) {
/*	    for (j=0; j<SL; j++) arrr1[SL*i+j]=arrr1[j];};
/*	    lmenu();
/*	    break;
/*	case '~':
/*	    for (i=1; i<16;  i++) {
/*	    for (j=0; j<20; j++) arrr1[20*i+j]=arrr1[j];};
/*	    lmenu();
/*	    break; */
        default: break;
        }; /* end switch */
    };	   /* end while */
}	   /* end main */

/* calculate one generation of evolution */
onehgen(arrr1) int  arrr1[QL][QL]; {int i0, i1, gn, arrr2[QL][QL];

/* count neighbors of main square */
phrame(1); /* phase to count neighbors */

for (i0=1; i0<QL-1; i0++) {
for (i1=1; i1<QL-1; i1++) {
  gn=arrr1[i0-1][i1];
  gn=KK*gn+arrr1[i0-1][i1-1];
  gn=KK*gn+arrr1[i0][i1-1];
  gn=KK*gn+arrr1[i0+1][i1];
  gn=KK*gn+arrr1[i0+1][i1+1];
  gn=KK*gn+arrr1[i0][i1+1];
  if (arrr1[i0][i1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[i0][i1]=gn;
  rs[gn]++;
  };};

/* count neighbors for top edge (i1=0) */

for (i0=1; i0<QL-1; i0++) {
  gn=arrr1[i0-1][0];
  gn=KK*gn+arrr1[i0-1][QL-1];
  gn=KK*gn+arrr1[i0][QL-1];
  gn=KK*gn+arrr1[i0+1][0];
  gn=KK*gn+arrr1[i0+1][1];
  gn=KK*gn+arrr1[i0][1];
  if (arrr1[i0][0]==1) gn+=GH;
  gn=gc[gn];
  arrr2[i0][0]=gn;
  rs[gn]++;
  };

/* count neighbors for bottom edge (i1=QL-1) */

for (i0=1; i0<QL-1; i0++) {
  gn=arrr1[i0-1][QL-1];
  gn=KK*gn+arrr1[i0-1][QL-2];
  gn=KK*gn+arrr1[i0][QL-2];
  gn=KK*gn+arrr1[i0+1][QL-1];
  gn=KK*gn+arrr1[i0+1][0];
  gn=KK*gn+arrr1[i0][0];
  if (arrr1[i0][QL-1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[i0][QL-1]=gn;
  rs[gn]++;
  };

/* count neighbors for left edge (i0=0) */

for (i1=1; i1<QL-1; i1++) {
  gn=arrr1[QL-1][i1];
  gn=KK*gn+arrr1[QL-1][i1-1];
  gn=KK*gn+arrr1[0][i1-1];
  gn=KK*gn+arrr1[1][i1];
  gn=KK*gn+arrr1[1][i1+1];
  gn=KK*gn+arrr1[0][i1+1];
  if (arrr1[0][i1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[0][i1]=gn;
  rs[gn]++;
  };

/* count neighbors of right edge (i0=QL-1) */

for (i1=1; i1<QL-1; i1++) {
  gn=arrr1[QL-2][i1];
  gn=KK*gn+arrr1[QL-2][i1-1];
  gn=KK*gn+arrr1[QL-1][i1-1];
  gn=KK*gn+arrr1[0][i1];
  gn=KK*gn+arrr1[0][i1+1];
  gn=KK*gn+arrr1[QL-1][i1+1];
  if (arrr1[QL-1][i1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[QL-1][i1]=gn;
  rs[gn]++;
  };

/* count neighbors for upper left hand corner */

  gn=arrr1[QL-1][0];
  gn=KK*gn+arrr1[QL-1][QL-1];
  gn=KK*gn+arrr1[0][QL-1];
  gn=KK*gn+arrr1[0][1];
  gn=KK*gn+arrr1[1][1];
  gn=KK*gn+arrr1[0][1];
  if (arrr1[0][0]==1) gn+=GH;
  gn=gc[gn];
  arrr2[0][0]=gn;
  rs[gn]++;

/* count neighbors for upper right hand corner */

  gn=arrr1[QL-2][0];
  gn=KK*gn+arrr1[QL-2][QL-1];
  gn=KK*gn+arrr1[QL-1][QL-1];
  gn=KK*gn+arrr1[0][0];
  gn=KK*gn+arrr1[0][1];
  gn=KK*gn+arrr1[QL-1][1];
  if (arrr1[QL-1][0]==1) gn+=GH;
  gn=gc[gn];
  arrr2[QL-1][0]=gn;
  rs[gn]++;

/* count neighbors for bottom left hand corner */

  gn=arrr1[QL-1][QL-1];
  gn=KK*gn+arrr1[QL-1][QL-2];
  gn=KK*gn+arrr1[0][QL-2];
  gn=KK*gn+arrr1[1][QL-1];
  gn=KK*gn+arrr1[1][0];
  gn=KK*gn+arrr1[0][0];
  if (arrr1[0][QL-1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[0][QL-1]=gn;
  rs[gn]++;

/* count neighbors for bottom right hand corner */

  gn=arrr1[QL-2][QL-1];
  gn=KK*gn+arrr1[QL-2][QL-2];
  gn=KK*gn+arrr1[QL-1][QL-2];
  gn=KK*gn+arrr1[0][QL-1];
  gn=KK*gn+arrr1[0][0];
  gn=KK*gn+arrr1[QL-1][0];
  if (arrr1[QL-1][QL-1]==1) gn+=GH;
  gn=gc[gn];
  arrr2[QL-1][QL-1]=gn;
  rs[gn]++;

phrame(2); /* phase to update cells */
for (i0=0; i0<QL; i0++) {
for (i1=0; i1<QL; i1++) {
  arrr1[i0][i1]=trule[arrr2[i0][i1]]-'0';
};};
phrame(3); /* phase to show results */
}	/* end onehgen */
    
/* display a screen of evolution */
screen(arrr1) int  arrr1[QL][QL]; {int i0, i1;
  for (i1=0; i1<QL; i1++) {
  for (i0=0; i0<QL; i0++) {
    adot(i0,i1,arrr1[i0][i1]);
    };};
}

/* display a frame around the cell array */
phrame(l) int l; {int j;
for (j=0; j<QL; j++) {
  adot(0,j,l);
  adot(QL-1,j,l);
  adot(j,0,l);
  adot(j,QL-1,l);
  };
}

/* make a frame for a graph 		 */
/* (x,y) = lower left corner; e.g. (0,0) */
/* n     = dimension of frame            */
gphram(x,y,n) int x, y, n; {int i;
for (i=0; i<=n; i++) {
  videodot(199-y-i,x,0);
  videodot(199-y-i,x+n,0);
  videodot(199-n-y,x+i,0);
  videodot(199-y,x+i,0);
  };
for (i=0; i<=n; i+=10) {
  videodot(199-y-i,x,3);
  videodot(199-y-i,x+n,3);
  videodot(199-n-y,x+i,3);
  videodot(199-y,x+i,3);
  };
}

/* put a diagonal in a graph */
ghdiag(x,y,n) int x, y, n; {int i;
for (i=0; 2*i<n; i++) videodot(199-y-2*i,x+2*i,3);
}

/* graph incidence of rule usage */
/* beware limited count capacity  */
rinci(j) int j; {int i; for (i=0; i<TWIC; i++) sdot(i,j,rs[i]==0?1:3);}

/* graph statistics of rule usage */
/* beware limited count capacity  */
/* large counts look negative     */
rstat() {int i, j, m, n; double s, t;
for (i=0, s=0.0; i<TWIC; i++) s+=(double)rs[i];
if  (s<1.0) {for (i=0; i<TWIC; i++) rs[i]=0; return;};
for (i=0; i<TWIC; i++) {
  t=333.3*(((double)rs[i])/s); /* full scale = 1/10 population */
  if (t>32000.0) t=50.0;
  n=(int)t;
  sdot(i,33,0);
  m=33; if (n>33) {n=33; m=34;};
  if (n>0) for (j=0; j<n; j++) sdot(i,j,1);
  for (j=n; j<m; j++) sdot(i,j,2);
  rs[i]=0;
  };
}

/* generate the Golay code table */
golta() {
int i, j, k;
int i0, i1, i2, i3, i4, i5;
k=0;
for (i5=0; i5<KK; i5++)
for (i4=0; i4<KK; i4++)
for (i3=0; i3<KK; i3++)
for (i2=0; i2<KK; i2++)
for (i1=0; i1<KK; i1++)
for (i0=0; i0<KK; i0++) {
  i=binq(i0,i1,i2,i3,i4,i5);
  j=binq(i1,i2,i3,i4,i5,i0);
  if (j<i) i=j;
  j=binq(i2,i3,i4,i5,i0,i1);
  if (j<i) i=j;
  j=binq(i3,i4,i5,i0,i1,i2);
  if (j<i) i=j;
  j=binq(i4,i5,i0,i1,i2,i3);
  if (j<i) i=j;
  j=binq(i5,i0,i1,i2,i3,i4);
  if (j<i) i=j;
  j=binq(i5,i4,i3,i2,i1,i0);
  if (j<i) i=j;
  j=binq(i4,i3,i2,i1,i0,i5);
  if (j<i) i=j;
  j=binq(i3,i2,i1,i0,i5,i4);
  if (j<i) i=j;
  j=binq(i2,i1,i0,i5,i4,i3);
  if (j<i) i=j;
  j=binq(i1,i0,i5,i4,i3,i2);
  if (j<i) i=j;
  j=binq(i0,i5,i4,i3,i2,i1);
  if (j<i) i=j;
  j=binq(i0,i1,i2,i3,i4,i5);
  if (i==j) {gc[i]=k; nr[k]=j; k++;} else {gc[j]=gc[i];};
  /*
  printf("  %2X, gc[%3d]=%3d, k=%3d, (%d,%d)\n",j,j,i,k,i0+i2+i4,i1+i3+i5);
  */
  }
/*
printf("no. cl. = %3d",k);
*/
}

/* calculate binary equivalent */
int binq(i0,i1,i2,i3,i4,i5)
int i0, i1, i2, i3, i4, i5; {
return i0+KK*(i1+KK*(i2+KK*(i3+KK*(i4+KK*i5))));}

/* calculate Golay class of a cell */
int golcl(i0,i1,arrr1) int  i0, i1, arrr1[QL][QL]; {int gn;
  gn=arrr1[i0-1][i1];
  gn=KK*gn+arrr1[i0-1][i1-1];
  gn=KK*gn+arrr1[i0][i1-1];
  gn=KK*gn+arrr1[i0+1][i1];
  gn=KK*gn+arrr1[i0+1][i1+1];
  gn=KK*gn+arrr1[i0][i1+1];
  if (arrr1[i0][i1]==1) gn+=GH;
  gn=gc[gn];
  return gn;
}

/* calculate the image of a neighborhood */
int golcln(i0,i1,i2,i3,i4,i5,i6) int  i0, i1, i2, i3, i4, i5, i6;
{return trule[gc[(((((i0*KK+i1)*KK+i2)*KK+i3)*KK+i4)*KK+i5)*KK+i6]]-'0';}

/* turn the golay rule number into an ascii rule number */
goltoasc() {
int i0, i1, i2, i3, i4, i5, i6;
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
  for (i5=0; i5<KK; i5++)
  for (i6=0; i6<KK; i6++)
    ascrule[i0][i1][i2][i3][i4][i5][i6]=golcln(i0,i1,i2,i3,i4,i5,i6)+'0';
}

/* generate the de Bruijn diagram of s strip */
rijn() {int i, j, i0, i1, i2, i3, i4, i5, i6;
i=0; j=0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++) {
  i++;
  if (golcln(i0,i1,i2,i3,i4,i5,i6)==i0) {
    j++;
    videocursor(0,8,3);
    printf("%X-%X\n",binq(i0,i3,i4,i5,0,0),binq(i1,i2,i0,i6,0,0));
    };
}
videocursor(0,9,3);
printf("%d links amongst %d max",j,i);
}

/* graph Bernstein polynomial */
bhgraf(x,y,k,n) int x, y, k, n; {int i; double hexbern(), en, p;
en=(double)(n);
for (i=0; i<n; i++) {
  p=((double)(i))/en;
videodot(199-y-(int)(en*hexbern(p,k)),x+i,1);
};
}

/* evaluate the nth degree Bernstein polynomial at point p */
double hexbern(p,n) double p; int n; {double q, s, x, r; int i;
if (p>0.99) return bp[n];
q=1.0-p; r=p/q; s=0.0; x=1.0;
for (i=0; i<n;  i++) x*=q;
for (i=0; i<=n; i++, x*=r) s+=bp[i]*x;
s*=0.9998;
return(s+0.0001);
}

/* adot(i,j,l) - place dot on automaton screen at (i,j) in color l */
adot(i,j,l) int i, j, l; {videodot(199-PLR-(QL-1-j),PLC+(QL-1-j)+2*i,l);}

/* bdot(i,j,l) - place dot on Bernstein screen at (i,j) in color l */
bdot(i,j,l) int i, j, l; {videodot(199-j,i,l);}

/* sdot(i,j,l) - place dot on statistics screen at (i,j) in color l */
sdot(i,j,l) int i, j, l; {videodot(199-10-j,25+4*i,l);}

/* clear the cell array */
zarry(arrr1) int  arrr1[QL][QL]; {int i0, i1;
for (i0=0; i0<QL; i0++) {
for (i1=0; i1<QL; i1++) {
  arrr1[i0][i1]=0;
  };};
}

/* display rule menu */
hrmenu() {
  videocursor(0,0,33);
  scrstr("HEXL21");
  scrrul();
  videocursor(0,TRR,TRC);
  }

/* display rule number by sum in color */
scrrulx() {int i, j; 
for (j=0; j<KK; j++) {
for (i=0; i<NWOC; i++) {
  videocursor(0,TRR+j,TRC+8+i);
  resch(j*NWOC+i);
  };};
}

/* display rule number as a hexadecimal */
hscrrul() {int i; long int k;
  for (i=0, k=0; i<TWIC; i++) k=KK*k+trule[i]-'0';
  videocursor(0,WRR,WRC);
  printf("        ");
  videocursor(0,WRR,WRC);
  printf("%7.7lX",k);
  videocursor(0,0,0);
  }

/* display Wolfram rule number of cross section */
wscrrul() {int i;
  i=trule[25]-'0';
  i=KK*i+trule[20]-'0';
  i=KK*i+trule[10]-'0';
  i=KK*i+trule[2] -'0';
  i=KK*i+trule[20]-'0';
  i=KK*i+trule[18]-'0';
  i=KK*i+trule[2] -'0';
  i=KK*i+trule[0] -'0';
  videocursor(0,WRR,WRC+9);
  printf("     ");
  videocursor(0,WRR,WRC+9);
  printf("[%2d]",i);
  videocursor(0,0,0);
  }

/* edit the rule */
edhrule() {char c; int i, j;
  j=0; i=0;
  while (0<1) {
    hscrrul();
    wscrrul();
    shon(j,nr[i]);
    videocursor(0,TRR+j,TRC+i+8);
    videoputc(trule[j*NWOC+i],3);
    videocursor(0,TRR+j,TRC+i+8);
    c=kbdin();
      if (c=='\015') break;				/* carriage return */
      switch (c) {
        case '0': case '1': 				/* state = 0,1 */
          trule[j*NWOC+i]=c;
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case ' ': case '\315':				/* space = advance */
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
        case '\010': case '\313':			/* backspace */
	  resch(j*NWOC+i);
          if (i>0) i--; else if(j>0) {i=NWOC-1; j--;};
	  break;
	case '<':					/* cursor to front */
	  resch(j*NWOC+i);
	  i=0; j=0;
	  break;
	case '*':					/* cursor to center */
	  resch(j*NWOC+i);
	  i=0; j=1;
	  break;
	case '>':					/* cursor to end */
	  resch(j*NWOC+i);
	  i=NWOC-1; j=KK-1;
	  break;
	case '\322':					/* insert */
	  arule[j*NWOC+i]='+';
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\323':					/* delete */
	  arule[j*NWOC+i]=' ';
	  resch(j*NWOC+i);
          i++; if (i==NWOC) {i=0; j++; if (j==KK) j=0;};
	  break;
	case '\013': case '\310':			/* up arrow */
	  resch(j*NWOC+i);
          if (j==0) j=KK; else j--;
	  break;
	case '\012': case '\320':			/* down arrow */
	  resch(j*NWOC+i);
          j++; if (j==KK) j=0;
	  break;
	default:
	  break;
          };
    };
eran();
}

/* restore character in edhrule */
resch(i) int i; {videoputc(trule[i],(arule[i]==' ')?1:2);}

/* show neighborhood in edhrule */
/* i = center cell */
/* j = Golay code  */
shon(i,j) int i, j; {int k;
  videocursor(0,NRR,NRC);
  videoputc('0'+i,1);
  videocursor(0,NRR,NRC+2);
  k=j; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC+1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR-1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR,NRC-2);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC-1);
  k=k/KK; videoputc('0'+k%KK,1);
  videocursor(0,NRR+1,NRC+1);
  videoputc('0'+k/KK,1);
}

/* erase the neighborhood in edhrule */
eran() {
  videocursor(0,NRR,NRC); videoputc(' ',1);
  videocursor(0,NRR,NRC+2); videoputc(' ',1);
  videocursor(0,NRR-1,NRC+1); videoputc(' ',1);
  videocursor(0,NRR-1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR,NRC-2); videoputc(' ',1);
  videocursor(0,NRR+1,NRC-1); videoputc(' ',1);
  videocursor(0,NRR+1,NRC+1); videoputc(' ',1);
}

/* edit the plane of cells */
edhline(arrr1) int arrr1[QL][QL]; {
char c;
int  i, j, k, arrr2[QL][QL];

for (i=0; i<QL; i++) for (j=0; j<QL; j++) arrr2[i][j]=0;

videoscroll(18,0,24,24,0,0);
videocursor(0,19,0);
scrstr("insert states with 0,1,.");
videocursor(0,20,0);
scrstr("move cursor using arrows");
videocursor(0,21,0);
scrstr("NSEW R CGO fgmMxz *<>^_");
videocursor(0,22,0);
scrstr("carriage return exits");

while (0<1) {
  adot(li,lj,3);
  videocursor(0,0,1);
  videoputc('?',1);
  c=kbdin();
  videocursor(0,0,1);
  videoputc(c,1);
  if (c == '\015') break;
  switch (c) {
    case '0': case '1':	case '.':	/* enter state */
      if (c=='.') c='1';
      i=c-'0';
      arrr1[li][lj]=i;
      adot(li,lj,i);
      break;
    case '\012': case '\320':		/* down    */
      adot(li,lj,arrr1[li][lj]);
      lj++; if (lj==QL) lj=0;
      break;
    case '\013': case '\310':		/* up      */
      adot(li,lj,arrr1[li][lj]);
      if (lj==0) lj=QL; lj--;
      break;
    case '\014': case '\315': case ' ':	/* forward */
      adot(li,lj,arrr1[li][lj]);
      li++; if (li==QL) li=0;
      break;
    case '\010': case '\313':		/* back    */
      adot(li,lj,arrr1[li][lj]);
      if (li==0) li=QL; li--;
      break;
    case '<':				/* left margin  */
      adot(li,lj,arrr1[li][lj]);
      li=0;
      break;
    case '>':				/* right margin */
      adot(li,lj,arrr1[li][lj]);
      li=QL-1;
      break;
    case '^':				/* top margin   */
      adot(li,lj,arrr1[li][lj]);
      lj=0;
      break;
   case '_':				/* bottom margin  */
      adot(li,lj,arrr1[li][lj]);
      lj=QL-1;
      break;
   case '*':				/* center cursor */
      adot(li,lj,arrr1[li][lj]);
      li=QL/2; 
      lj=QL/2;
      break;
    case 'a':				/* insert glider */
      lcell(0,2);
      lcell(1,1); lcell(1,3);
      lcell(3,2);
      lcell(4,0); lcell(4,1);
      screen(arrr1);
      break; 
    case 'b':				/* insert glider */
      lcell(0,0);
      lcell(1,0);
      lcell(2,-1);
      lcell(3,0); lcell(3,-1);
      lcell(4,-2);
      screen(arrr1);
      break; 
     case 'f':				/* generate a frame */
      phrame (2);
      break;
    case 'g':				/* one cycle of evolution */
      phrame (1);
      onehgen(arrr1);
      screen(arrr1);
      break;
 case 'm':				/* mark neighborhood */
      arule[golcl(li,lj,arrr1)]='+';
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'M':				/* mark neighborhood over a region */
      if (li<10||li>QL-11) break;
      if (lj<10||lj>QL-11) break;
      for (i=li-10; i<li+10; i++) {
      for (j=lj-10; j<lj+10; j++) {
        arule[golcl(i,j,arrr1)]='+';
        };};
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'x':				/* uncommitted transitions */
      for (k=0; k<TWIC; k++) arule[k]=' ';
      videocursor(0,TRR,TRC+8);
      scrrul();
      break;
    case 'z':				/* clear array  */
      zarry(arrr1);
      screen(arrr1);
      break;
    case 'N':				/* rotate array up */
      for (i=0; i<QL; i++) {
        k=arrr1[i][0]; 
        for (j=1; j<QL; j++) arrr1[i][j-1]=arrr1[i][j];
	arrr1[i][QL-1]=k;
	};
      if (lj==0) lj=QL; lj--;
      screen(arrr1);
      break;
    case 'S':				/* rotate array down */
      for (i=0; i<QL; i++) {
	k=arrr1[i][QL-1];
        for (j=1; j<QL; j++) arrr1[i][QL-j]=arrr1[i][QL-j-1];
	arrr1[i][0]=k;
	};
      lj++; if (lj==QL) lj=0;
      screen(arrr1);
      break;
    case 'E':				/* rotate array right */
      for (j=0; j<QL; j++) {
	k=arrr1[QL-1][j];
        for (i=1; i<QL; i++) arrr1[QL-i][j]=arrr1[QL-i-1][j];
	arrr1[0][j]=k;
	};
      li++; if (li==QL) li=0;
      screen(arrr1);
      break;
    case 'W':				/* rotate array left */
      for (j=0; j<QL; j++) {
	k=arrr1[0][j];
        for (i=1; i<QL; i++) arrr1[i-1][j]=arrr1[i][j];
	arrr1[QL-1][j]=k;
	};
      if (li==0) li=QL; li--;
      screen(arrr1);
      break;
    case 'R':				/* reflect array in diagonal */
      for (j=1; j<QL; j++) {
      for (i=0; i<j;  i++) {
	k=arrr1[i][j];
	arrr1[i][j]=arrr1[j][i];
        arrr1[j][i]=k;
        };};
      k=li; li=lj; lj=k;
      screen(arrr1);
      break;
    case 'C':				/* save cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arrr2[i][j]=arrr1[i][j];
	};};
      break;
    case 'G':				/* recover saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arrr1[i][j]=arrr2[i][j];
	};};
      screen(arrr1);
      break;
    case 'O':				/* or saved cell array */
      for (j=0; j<QL; j++) {
      for (i=0; i<QL; i++) {
	arrr1[i][j]|=arrr2[i][j];
	};};
      screen(arrr1);
      break;
    default:
      break;
      };				/* end switch */
  };					/* end while */
adot(li,lj,arrr1[li][lj]);
videoscroll(18,0,24,24,0,0);
videocursor(0,0,1);
videoputc(' ',1);
}				        /* end edline */

/* generate the transitions for Golay's Life */
conlif() {int i;
printf("Golay's Life\n");
    for (i=0; i<TWIC; i++) trule[i]='0';
    trule[2]='1';
    trule[3]='1';
    trule[15]='1';
    trule[16]='1';
}

/* generate random cells in arrr1 */
ranarr(arrr1) int  arrr1[QL][QL]; {int i0, i1, i, c;
i=0;
for (i0=0; i0<QL; i0++) {
for (i1=0; i1<QL; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arrr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate random cells in the center of arrr1 */
rancen(arrr1) int  arrr1[QL][QL]; {int i0, i1, ia, ib, i, c;
i=0;
for (i0=0; i0<QL; i0++) {
for (i1=0; i1<QL; i1++) {
  arrr1[i0][i1]=0;
  };};
ia=QL/3; ib=2*(QL/3);
for (i0=ia; i0<ib; i0++) {
for (i1=ia; i1<ib; i1++) {
  if (i%8 == 0) c=rand(); i++;
  arrr1[i0][i1]=c%KK; c/=KK;
  };};
}

/* generate a random rule */
ranhrul() {int i0, i, c;
i=0;
for (i0=0; i0<TWIC; i0++) {
  if (i%8 == 0) c=rand(); i++;
  trule[i0]=c%KK+'0'; c/=KK;
};
}

# include "nxbrui21.c"

/* end nxhexl21.c */
