

/* probks.c						*/
/* program for LCAU2S option 't'			*/
/* calculate probabilities related to evolution		*/
/* generic portion of probkr.c which applies to radius  */
/* 7/2 automata and which do not make explicit use of   */
/* the number of states. It is to be included in each   */
/* specific probks file.				*/
/* Harold V. McIntosh, 20 January 1992			*/

/*  Copyright (C) 1992  */
/*  Harold V. McIntosh  */

/* ---------------------------------------------------- */

/* gather totalistic neighborhood sums for one generation */
/* mm = number of cells					  */
/* b[cc]  = sum counts					  */

onesum(mm,cc,b) int mm, cc, *b; {int i, j, k;
  for (i=0; i<cc; i++) b[i]=0;
  k=arr1[mm-3]+arr1[mm-2]+arr1[mm-1]
	+arr1[0]+arr1[1]+arr1[2]+arr1[3]+arr1[4]; b[k]++;
  k=arr1[mm-2]+arr1[mm-1]+arr1[0]
	+arr1[1]+arr1[2]+arr1[3]+arr1[4]+arr1[5]; b[k]++;
  k=arr1[mm-1]+arr1[0]+arr1[1]
	+arr1[2]+arr1[3]+arr1[4]+arr1[5]+arr1[6]; b[k]++;
  k=arr1[0]+arr1[1]+arr1[2]
	+arr1[3]+arr1[4]+arr1[5]+arr1[6]+arr1[7]; b[k]++;
  for (i=4; i<mm-4; i++) {k=0; for (j=i-3; j<i+5; j++) k+=arr1[j]; b[k]++;}
  k=arr1[mm-7]+arr1[mm-6]+arr1[mm-5]+arr1[mm-4]
	+arr1[mm-3]+arr1[mm-2]+arr1[mm-1]+arr1[0]; b[k]++;
  k=arr1[mm-6]+arr1[mm-5]+arr1[mm-4]+arr1[mm-3]
	+arr1[mm-2]+arr1[mm-1]+arr1[0]+arr1[1]; b[k]++;
  k=arr1[mm-5]+arr1[mm-4]+arr1[mm-3]+arr1[mm-2]
	+arr1[mm-1]+arr1[0]+arr1[1]+arr1[2]; b[k]++;
  k=arr1[mm-4]+arr1[mm-3]+arr1[mm-2]+arr1[mm-1]
	+arr1[0]+arr1[1]+arr1[2]+arr1[3]; b[k]++;
}

/* Generate coefficients of 2nd generation Bernstein Polynomial */

berncoef(f) double *f; {
int i, i0, i1, i2, i3, i4, i5, i6, i7;

for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
  if (ascrule[i0][i1][i2][i3][i4][i5][i6][i7]=='1')
	 bp[i0+i1+i2+i3+i4+i5+i6+i7]+=1.0;
for (i=0; i<BD; i++) bp[i]*=f[i];
}

/* Generate coefficients of 3rd generation Bernstein Polynomial */

bernthrd() {
int i, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic, id, ie;
int j0, j1, j2, j3, j4, j5, j6, j7;

asctobin();
for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++)
for (id=0; id<KK; id++)
for (ie=0; ie<KK; ie++) {
  j0=binrule[i0][i1][i2][i3][i4][i5][i6][i7];
  j1=binrule[i1][i2][i3][i4][i5][i6][i7][i8];
  j2=binrule[i2][i3][i4][i5][i6][i7][i8][i9];
  j3=binrule[i3][i4][i5][i6][i7][i8][i9][ia];
  j4=binrule[i4][i5][i6][i7][i8][i9][ia][ib];
  j5=binrule[i5][i6][i7][i8][i9][ia][ib][ic];
  j6=binrule[i6][i7][i8][i9][ia][ib][ic][id];
  j7=binrule[i7][i8][i9][ia][ib][ic][id][ie];
  i=i0+i1+i2+i3+i4+i5+i6+i7+i8+i9+ia+ib+ic+id+ie;
  if (ascrule[j0][j1][j2][j3][j4][j5][j6][j7]=='1') bp[i]+=1.0;
  }
}

/* Generate coefficients of 4th generation Bernstein Polynomial */

bernfrth() {
int i, i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib, ic, id, ie;
int    jf, ig, ih, ii, ij, ik, il;
int j0, j1, j2, j3, j4, j5, j6, j7, j8, j9, ja, jb, jc, jd, je;
int k0, k1, k2, k3, k4, k5, k6, k7;

asctobin();
for (i=0; i<BD; i++) bp[i]=0.0;
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++)
for (ic=0; ic<KK; ic++)
for (id=0; id<KK; id++)
for (ie=0; ie<KK; ie++)
for (jf=0; jf<KK; jf++)
for (ig=0; ig<KK; ig++)
for (ih=0; ih<KK; ih++)
for (ii=0; ii<KK; ii++)
for (ij=0; ij<KK; ij++)
for (ik=0; ik<KK; ik++)
for (il=0; il<KK; il++) {
  j0=binrule[i0][i1][i2][i3][i4][i5][i6][i7];
  j1=binrule[i1][i2][i3][i4][i5][i6][i7][i8];
  j2=binrule[i2][i3][i4][i5][i6][i7][i8][i9];
  j3=binrule[i3][i4][i5][i6][i7][i8][i9][ia];
  j4=binrule[i4][i5][i6][i7][i8][i9][ia][ib];
  j5=binrule[i5][i6][i7][i8][i9][ia][ib][ic];
  j6=binrule[i6][i7][i8][i9][ia][ib][ic][id];
  j7=binrule[i7][i8][i9][ia][ib][ic][id][ie];
  j8=binrule[i8][i9][ia][ib][ic][id][ie][jf];
  j9=binrule[i9][ia][ib][ic][id][ie][jf][ig];
  ja=binrule[ia][ib][ic][id][ie][jf][ig][ih];
  jb=binrule[ib][ic][id][ie][jf][ig][ih][ii];
  jc=binrule[ic][id][ie][jf][ig][ih][ii][ij];
  jd=binrule[id][ie][jf][ig][ih][ii][ij][ik];
  je=binrule[ie][jf][ig][ih][ii][ij][ik][il];

  k0=binrule[j0][j1][j2][j3][j4][j5][j6][j7];
  k1=binrule[j1][j2][j3][j4][j5][j6][j7][j8];
  k2=binrule[j2][j3][j4][j5][j6][j7][j8][j9];
  k3=binrule[j3][j4][j5][j6][j7][j8][j9][ja];
  k4=binrule[j4][j5][j6][j7][j8][j9][ja][jb];
  k5=binrule[j5][j6][j7][j8][j9][ja][jb][jc];
  k6=binrule[j6][j7][j8][j9][ja][jb][jc][jd];
  k7=binrule[j7][j8][j9][ja][jb][jc][jd][je];

  i=i0+i1+i2+i3+i4+i5+i6+i7+i8+i9+ia+ib+ic+id+ie+jf+ig+ih+ii+ij+ik+il;
  if (ascrule[k0][k1][k2][k3][k4][k5][k6][k7]=='1') bp[i]+=1.0;
  }
}

asfreq(l) int l; {
int    i, j, i0, i1, i2, i3, i4, i5, i6, i7;
int    stal4, stal3, stal2, stal1;
int    stac;
int    star1, star2, star3, star4;	/* statistic counts */
int    stat[KK];
double staf[KK], t, pp;

videoscroll(BROW,0,BROW+8,40,0,0);
videocursor(0,BROW,0);
pp=1.0/((double)(KK*KK*KK*KK*KK*KK*KK*KK));
stal4=stal3=stal2=stal1=0;
stac=0;
star1=star2=star3=star4=0;
for (i=0; i<KK; i++) stat[i]=0;
for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
for (i2=0; i2<KK; i2++) {
for (i3=0; i3<KK; i3++) {
for (i4=0; i4<KK; i4++) {
for (i5=0; i5<KK; i5++) {
for (i6=0; i6<KK; i6++) {
for (i7=0; i7<KK; i7++) {
  j=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  stat[j]++;
  if(j==i0) star4++;
  if(j==i1) star3++;
  if(j==i2) star2++;
  if(j==i3) star1++;
  if(j==i4) stac++;
  if(j==i5) stal1++;
  if(j==i6) stal2++;
  if(j==i7) stal3++;
  }}}}}}}}
for (i=0; i<KK; i++) printf("%2d    - %5.2f\n",i,((double)stat[i])*pp);
printf("\n");
printf("4left  - %5.2f      4right  - %5.2f\n",((double)stal4)*pp,((double)star4)*pp);
printf("3left  - %5.2f      3right  - %5.2f\n",((double)stal3)*pp,((double)star3)*pp);
printf("2left  - %5.2f      2right  - %5.2f\n",((double)stal2)*pp,((double)star2)*pp);
printf("1left  - %5.2f      1right  - %5.2f\n",((double)stal1)*pp,((double)star1)*pp);
printf("still  - %5.2f\n",((double)stac)*pp);
for (i=0; i<KK; i++) staf[i]=((double)stat[i])*pp;
j=199-(int)(100.0*staf[1]);
i=CORG+(int)(100.0*staf[1]);
if (j>=2) videodot(j-2,i,l);
if (i>=2) videodot(j,i-2,l);
videodot(j,i,1);
if (i<=318) videodot(j,i+2,l);
if (j<=198) videodot(j+2,i,l);
for (i=0, t=0.0; i<KK; i++) t+=staf[i]*log(staf[i]);
videocursor(0,BROW,20);
printf("lambda:   %5.3f",1.0-staf[0]);
videocursor(0,BROW+1,20);
printf("entropy:  %5.3f",-t/log((double)KK));
}

/* evaluate the one-block probabilities after one generation */
onebl(x,a) double x[KK], a[KK]; {
int i0, i1, i2, i3, i4, i5, i6, i7, j0;

for (j0=0; j0<KK; j0++) x[j0]=0.0;

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  x[j0]+=a[i0]*a[i1]*a[i2]*a[i3]*a[i4]*a[i5]*a[i6]*a[i7];
  }
}

/* iterate the 1-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* mm - length of line	*/
/* nn - number of lines	*/
oneblfreq(mm,nn) int mm, nn; {
int    ii, i, l, m, n;
double op[KK], np[KK];
double d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++) {op[i]=s; for (l=0; l<n; l++) videodot(0,m++,i);}

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  onebl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {
    n=(int)(f*np[i]); if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i);
    d=op[i]-np[i];
    e+=d*d;
    op[i]=np[i];
    }
  videodot(ii,m++,3);
  if (op[0]<=0.001) break;
  if (op[1]<=0.001) break;
  if (e<=0.0000001) break;
  }
videocursor(0,BROW+7,0);
printf("(1-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,op[i]);
}

/* evaluate the two-block probabilities after one generation */
twobl(x,a) double x[KK][KK], a[KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8;
int    j0, j1;
double w, s, b[KK];

for (j0=0; j0<KK; j0++) {for (j1=0; j1<KK; j1++) x[j0][j1]=0.0;}

for (j0=0; j0<KK; j0++) {b[j0]=0.0; for (j1=0; j1<KK; j1++) b[j0]+=a[j0][j1];}

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  j1=ascrule[i1][i2][i3][i4][i5][i6][i7][i8]-'0';
  w=a[i0][i1]*a[i1][i2]*a[i2][i3]*a[i3][i4]*a[i4][i5]
	*a[i5][i6]*a[i6][i7]*a[i7][i8];
  s=b[i1]*b[i2]*b[i3]*b[i4]*b[i5]*b[i6]*b[i7];
  if (s!=0.0) w/=s;
  x[j0][j1]+=w;
  }
}

/* iterate the 2-block parameters to find self-consistent values */
/* graph the iterative steps in bar-chart form			 */
/* mm - length of line	*/
/* nn - number of lines	*/
twoblfreq(mm,nn) int mm, nn; {
int    ii, i, j, l, m, n;
double op[KK][KK], np[KK][KK];
double b[KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  op[i][j]=s;
  for (l=0; l<n; l++) videodot(0,m++,j);
  }}
videodot(0,m++,3);

for (i=0; i<KK; i++) {b[i]=0.0; for (j=0; j<KK; j++) b[i]+=op[i][j];}
videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),1);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  twobl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
    n=(int)(f*np[i][j]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,j);
    d=op[i][j]-np[i][j];
    e+=d<0.0?-d:d;
    op[i][j]=np[i][j];
    }}
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {b[i]=0.0; for (j=0; j<KK; j++) b[i]+=op[i][j];}
  videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),2);
  if (e<=0.0001) break;
  }
videocursor(0,BROW+7,0);
printf("(2-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,b[i]);
videocursor(0,BROW+8,0);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
		printf("%1d%1d:%5.3f ",i,j,op[i][j]);
videodot(199-(int)(100.0*op[1][1]),(int)(100.0*b[1]),3);
videodot(199-(int)(100.0*op[1][1]),BORG+(int)(100.0*b[1]),1);
}

/* evaluate the three-block probabilities after one generation */
thrbl(x,a) double x[KK][KK][KK], a[KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8, i9;
int    j0, j1, j2;
double w, b[KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
  for (j2=0; j2<KK; j2++) x[j0][j1][j2]=0.0;
  b[j0][j1]=0.0;
  for (j2=0; j2<KK; j2++) b[j0][j1]+=a[j0][j1][j2];  
  }}

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  j1=ascrule[i1][i2][i3][i4][i5][i6][i7][i8]-'0';
  j2=ascrule[i2][i3][i4][i5][i6][i7][i8][i9]-'0';
  w=a[i0][i1][i2]*a[i1][i2][i3]*a[i2][i3][i4]*a[i3][i4][i5]
	*a[i4][i5][i6]*a[i5][i6][i7]*a[i6][i7][i8]*a[i7][i8][i9];
  if (w!=0.0) w/=b[i1][i2]*b[i2][i3]*b[i3][i4]*b[i4][i5]
	*b[i5][i6]*b[i6][i7]*b[i7][i8];
  x[j0][j1][j2]+=w;
  }
}

/* iterate the 3-block parameters to find self-consistent values */
/* mm - length of line	*/
/* nn - number of lines	*/
thrblfreq(mm,nn) int mm, nn; {
int    ii, i, j, k, l, m, n;
double op[KK][KK][KK], np[KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i=0; i<KK; i++)
for (j=0; j<KK; j++)
for (k=0; k<KK; k++) {
  op[i][j][k]=s;
  for (l=0; l<n; l++) videodot(0,m++,k);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  thrbl(np,op);
  videodot(ii,m++,3);
  for (i=0; i<KK; i++) {
  for (j=0; j<KK; j++) {
  for (k=0; k<KK; k++) {
    n=(int)(f*np[i][j][k]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,k);
    d=op[i][j][k]-np[i][j][k];
    e+=d<0.0?-d:d;
    op[i][j][k]=np[i][j][k];
    }}}
  videodot(ii,m++,3);
  if (e<=0.0001) break;
  }

for (i=0; i<KK; i++) {
  b[i]=0.0;
  for (j=0; j<KK; j++) {
  for (k=0; k<KK; k++) {
    b[i]+=op[i][j][k];
    }} }
videocursor(0,BROW+7,0);
printf("(3-block) "); 
for (i=0; i<KK; i++) printf("%2d:%5.3f ",i,b[i]);

for (i=0; i<KK; i++) {
for (j=0; j<KK; j++) {
  bb[i][j]=0.0;
  for (k=0; k<KK; k++) {
    bb[i][j]+=op[i][j][k];
    }} }

videocursor(0,BROW+8,0);
for (i=0; i<KK; i++) for (j=0; j<KK; j++) 
  printf("%1d%1d:%5.3f ",i,j,bb[i][j]);
videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);
}

/* evaluate the four-block probabilities after one generation */
foubl(x,a) double x[KK][KK][KK][KK], a[KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia;
int    j0, j1, j2, j3;
double w, b[KK][KK][KK];

for (j0=0; j0<KK; j0++) {
for (j1=0; j1<KK; j1++) {
for (j2=0; j2<KK; j2++) {
  for (j3=0; j3<KK; j3++) x[j0][j1][j2][j3]=0.0;
  b[j0][j1][j2]=0.0;
  for (j3=0; j3<KK; j3++) b[j0][j1][j2]+=a[j0][j1][j2][j3];  
  }}}

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  j1=ascrule[i1][i2][i3][i4][i5][i6][i7][i8]-'0';
  j2=ascrule[i2][i3][i4][i5][i6][i7][i8][i9]-'0';
  j3=ascrule[i3][i4][i5][i6][i7][i8][i9][ia]-'0';
  w=a[i0][i1][i2][i3];
  w*=a[i1][i2][i3][i4];
  w*=a[i2][i3][i4][i5];
  w*=a[i3][i4][i5][i6];
  w*=a[i4][i5][i6][i7];
  w*=a[i5][i6][i7][i8];
  w*=a[i6][i7][i8][i9];
  w*=a[i7][i8][i9][ia];
  if (w!=0.0) w/=b[i1][i2][i3]*b[i2][i3][i4]*b[i3][i4][i5]
	*b[i4][i5][i6]*b[i5][i6][i7]*b[i6][i7][i8]*b[i7][i8][i9];
  x[j0][j1][j2][j3]+=w;
  }
}

/* iterate the 4-block parameters to find self-consistent values */
/* mm - length of line	*/
/* nn - number of lines	*/
foublfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, l, m, n;
double op[KK][KK][KK][KK], np[KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++) {
  op[i0][i1][i2][i3]=s;
  for (l=0; l<n; l++) videodot(0,m++,i3);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  foubl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++) {
    n=(int)(f*np[i0][i1][i2][i3]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i3);
    d=op[i0][i1][i2][i3]-np[i0][i1][i2][i3];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3]=np[i0][i1][i2][i3];
    }
  videodot(ii,m++,3);
  if (e<=0.0001) break;
  }

for (i0=0; i0<KK; i0++) {
  b[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
    b[i0]+=op[i0][i1][i2][i3];
  }

videocursor(0,BROW+7,0);
printf("(4-block) "); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,b[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
    bb[i0][i1]+=op[i0][i1][i2][i3];
    }} }}

videocursor(0,BROW+8,0);
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);
}

/* evaluate the five-block probabilities after one generation */
fivbl(x,a) double x[KK][KK][KK][KK][KK], a[KK][KK][KK][KK][KK]; {
int    i0, i1, i2, i3, i4, i5, i6, i7, i8, i9, ia, ib;
int    j0, j1, j2, j3, j4;
double w, b[KK][KK][KK][KK];

for (j0=0; j0<KK; j0++)
for (j1=0; j1<KK; j1++)
for (j2=0; j2<KK; j2++)
for (j3=0; j3<KK; j3++) {
  for (j4=0; j4<KK; j4++) x[j0][j1][j2][j3][j4]=0.0;
  b[j0][j1][j2][j3]=0.0;
  for (j4=0; j4<KK; j4++) b[j0][j1][j2][j3]+=a[j0][j1][j2][j3][j4];  
  }

for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++)
for (i5=0; i5<KK; i5++)
for (i6=0; i6<KK; i6++)
for (i7=0; i7<KK; i7++)
for (i8=0; i8<KK; i8++)
for (i9=0; i9<KK; i9++)
for (ia=0; ia<KK; ia++)
for (ib=0; ib<KK; ib++) {
  j0=ascrule[i0][i1][i2][i3][i4][i5][i6][i7]-'0';
  j1=ascrule[i1][i2][i3][i4][i5][i6][i7][i8]-'0';
  j2=ascrule[i2][i3][i4][i5][i6][i7][i8][i9]-'0';
  j3=ascrule[i3][i4][i5][i6][i7][i8][i9][ia]-'0';
  j4=ascrule[i4][i5][i6][i7][i8][i9][ia][ib]-'0';
  w=a[i0][i1][i2][i3][i4];
  w*=a[i1][i2][i3][i4][i5];
  w*=a[i2][i3][i4][i5][i6];
  w*=a[i3][i4][i5][i6][i7];
  w*=a[i4][i5][i6][i7][i8];
  w*=a[i5][i6][i7][i8][i9];
  w*=a[i6][i7][i8][i9][ia];
  w*=a[i7][i8][i9][ia][ib];
  if (w!=0.0) w/=b[i1][i2][i3][i4]*b[i2][i3][i4][i5]*b[i3][i4][i5][i6]
	*b[i4][i5][i6][i7]*b[i5][i6][i7][i8]*b[i6][i7][i8][i9]
	*b[i7][i8][i9][ia];
  x[j0][j1][j2][j3][j4]+=w;
  }
}

/* iterate the 5-block parameters to find self-consistent values */
/* mm - length of line	*/
/* nn - number of lines	*/
fivblfreq(mm,nn) int mm, nn; {
int    ii, i0, i1, i2, i3, i4, l, m, n;
double op[KK][KK][KK][KK][KK], np[KK][KK][KK][KK][KK];
double b[KK], bb[KK][KK], d, e, f, s;

m=0;
f=(double)mm;
s=1.0/((double)(KK*KK*KK*KK*KK));
n=(int)(f*s);
videodot(0,m++,3);
for (i0=0; i0<KK; i0++)
for (i1=0; i1<KK; i1++)
for (i2=0; i2<KK; i2++)
for (i3=0; i3<KK; i3++)
for (i4=0; i4<KK; i4++) {
  op[i0][i1][i2][i3][i4]=s;
  for (l=0; l<n; l++) videodot(0,m++,i4);
  }
videodot(0,m++,3);

for (ii=1; ii<=nn; ii++) {
  e=0.0; m=0;
  fivbl(np,op);
  videodot(ii,m++,3);
  for (i0=0; i0<KK; i0++)
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++) {
    n=(int)(f*np[i0][i1][i2][i3][i4]);
    if (n>0) for (l=0; l<n; l++) videodot(ii,m++,i4);
    d=op[i0][i1][i2][i3][i4]-np[i0][i1][i2][i3][i4];
    e+=d<0.0?-d:d;
    op[i0][i1][i2][i3][i4]=np[i0][i1][i2][i3][i4];
    }
  videodot(ii,m++,3);
  if (e<=0.0001) break;
  }
for (i0=0; i0<KK; i0++) {
  b[i0]=0.0;
  for (i1=0; i1<KK; i1++)
  for (i2=0; i2<KK; i2++)
  for (i3=0; i3<KK; i3++)
  for (i4=0; i4<KK; i4++)
    b[i0]+=op[i0][i1][i2][i3][i4];
  }
videocursor(0,BROW+7,0);
printf("(5-block) "); 
for (i0=0; i0<KK; i0++) printf("%2d:%5.3f ",i0,b[i0]);

for (i0=0; i0<KK; i0++) {
for (i1=0; i1<KK; i1++) {
  bb[i0][i1]=0.0;
  for (i2=0; i2<KK; i2++) {
  for (i3=0; i3<KK; i3++) {
  for (i4=0; i4<KK; i4++) {
    bb[i0][i1]+=op[i0][i1][i2][i3][i4];
    }} }}}

videocursor(0,BROW+8,0);
for (i0=0; i0<KK; i0++) for (i1=0; i1<KK; i1++) 
  printf("%1d%1d:%5.3f ",i0,i1,bb[i0][i1]);
videodot(199-(int)(100.0*bb[1][1]),BORG+(int)(100.0*b[1]),2);
}

sixblfreq(mm,nn) int mm, nn; {}
svnblfreq(mm,nn) int mm, nn; {}

/* end probks.c */
